/*
 * gpio.cpp
 *
 * Created: 09/12/2014 17:19:34
 *  Author: Reta
 */ 

#include "gpio.h"

gpio::gpio()
{
	
}

gpio::gpio(volatile uint8_t *DDR, volatile uint8_t *PORT, uint8_t PIN)
{
	ddr = DDR;		// Data Direccion Register
	port = PORT;	// Port Data Register
	pin = PIN;		// Pin selected
	
	isIntEnable = false;
}

void gpio::reset()
{
	*ddr &= ~(1 << pin);  // pin of DDR is low
	*port &= ~(1 << pin); // pin of PORT is low
	
	if(isIntEnable)
	{
		*imsk &= ~(1 << pin);			// disable interrupt on pin of port
		isIntEnable = false;
	}
	
}

void gpio::output()
{
	*ddr |= (1 << pin); // pin of port is set as output
	low();
}

void gpio::input(volatile uint8_t * IN, bool pull_up)
{	
	port_in = IN;						// Port In Data
	*ddr &= ~(1 << pin);				// pin of port is set as input
	if(pull_up) *port |= (1 << pin);	// enable pull-up resistor	
}

void gpio::input(volatile uint8_t * IN, bool pull_up,  volatile uint8_t * IMSK)
{	
	port_in = IN;						// Port In Data
	imsk = IMSK;						// Mask Register
	
	*ddr &= ~(1 << pin);				// pin of port is set as input
	if(pull_up) *port |= (1 << pin);	// enable pull-up resistor	
}

void gpio::set_interrupt_enable(bool enabled)
{
	if(enabled)
	{
		*imsk |= (1 << pin);			// enable interrupt on pin of port
		isIntEnable = true;
	}
	else
	{
		*imsk &= ~(1 << pin);			// disable interrupt on pin of port
		isIntEnable = false;
	}
}

bool gpio::read()
{
	if(*port_in & (1 << pin)) return true;	// if pin is 1 return true
	else				 return false;	// if pin is 0 return false
}

void gpio::high()
{
	*port |= (1 << pin); // pin of PORT is high
}

void gpio::low()
{
	*port &= ~(1 << pin); // pin of PORT is low
}

void gpio::toggle()
{
	*port ^= (1 << pin);
}
