/*
 * uart.cpp
 *
 * Created: 09/12/2014 16:21:47
 *  Author: Reta
 */ 

#include "uart.h"

// Atmel Studio 6.2 :
// Para usar sprintf con double o float, hacer:
// En propiedades del proyecto, toolchain, AVR/GNU Linker:
//															* General: check "Use vprintf" box
//															* Libraries: add "libprintf_flt.a". Pon libm como ltimo
// Ms info en la siguiente pgina-> http://www.avrfreaks.net/forum/question-about-using-vsprintf-function


Uart::Uart()
{
	
}

bool	Uart::IsRx = false;
char	Uart::Rx = 0;
const uint8_t Uart::Tx_Length = 150;
char 	Uart::Tx[Uart::Tx_Length] = {0};

void Uart::setup(uint32_t baud, uint32_t frecuency_cpu)
{ 
	// Set baud rate
	uint16_t My_UBRR = frecuency_cpu/16/baud -1;
	
	UBRR0H = (My_UBRR >> 8) & 0xff; 
	UBRR0L = My_UBRR & 0xff;
	UCSR0B |= (1 << RXEN0) | (1 << TXEN0);	// Enable receiver and transmitter
	UCSR0B |= (1 << RXCIE0);				// Enable receiver interrupt
	UCSR0C |= (1 << UCSZ01) | (1 << UCSZ00); // Set frame: 8data, 1 stop
}

void Uart::reset_variable()
{
	Uart::IsRx = false;
	Uart::Rx = 0;
}

void Uart::off()
{
	UBRR0H = 0;
	UBRR0L = 0;
	UCSR0B = 0;
	UCSR0C = 0;
}

void Uart::Send_byte(uint8_t data)
{
	while ( !( UCSR0A & (1<<UDRE0)) ) // wait for empty transmit buffer
	;
	//UDR0 = (char)data; // put data into	buffer, sends the data
	UDR0 = data; // put data into	buffer, sends the data
}

void Uart::Send_Word(uint16_t data)
{
	Send_byte((data >> 8) & 0xff);
	Send_byte(data & 0xff);
}

void Uart::Send_string(const char *str)
{
	while (*str) // while the value of str was different to 0 continue
	{
		Send_byte(*str++); // send the char of address str.Then plus 1 to address str
	}
}

void Uart::Send_int(uint16_t d)
{
	char str[6]; // create a array of char = strings
	//sprintf(str,"%.4u",d); // convert of int to string, with a minimum of 4 digits. adding zeros if its necessary
	sprintf(str,"%u\n",d); // convert of int to string
	Send_string(str); // send strings
}

void Uart::Send_double(double d)
{
	char str[6]; // create a array of char = strings
	sprintf(str,"%.2f\n",d); // convert of double to string, with a width of 2 after The point, adding zeros if its necessary
	Send_string(str); // send strings
}

uint8_t Uart::Receive()
{
	while ( !(UCSR0A & (1<<RXC0)) ) // wait for data to be received
	;
	return UDR0; // get and return received data from buffer
}

int Uart::Receive_until(char * r_aux, char final_char)
{
	int i = 0;
	Uart::Rx = 0;
	Uart::IsRx = false;
	
	while(true)
	{
		_delay_us(1);		
		
		if(Uart::IsRx)
		{
			//Send_string(rx);
			if(Uart::Rx == final_char)
			break;
			
			r_aux[i] = Rx;
			i++;
			Uart::Rx = 0;
			Uart::IsRx = false;
		}
	}
	
	return i;
}

uint8_t Uart::Receive_int_until(char final_char)
{
	char r_aux [10] = {0};
	int length = Receive_until(r_aux, final_char);
	r_aux[length] = '\0';
	return (uint8_t)atoi(r_aux);
	
}

float Uart::Receive_float_until(char final_char)
{
	char r_aux [10] = {0};
	int length = Receive_until(r_aux, final_char);	
	r_aux[length] = '\0';
	return atof(r_aux);
}

void Uart::Rx_Interrupt_Handler()
{	
	if(!Uart::IsRx)	
	{
		Uart::Rx = Receive(); // Read data from the RX buffer
		Uart::IsRx = true;
	}	
}