/*
 * Plataforma_Movil_Final.cpp
 *
 * Created: 21/07/2015 14:56:36
 *  Author: Reta
 */ 


// Revisar la adicin del metodo cli() and sei()


#include "Clases/constantes.h"
#include "Clases/init.h"

#include "Clases/Adc/adc.h"
#include "Clases/Gpio/gpio.h"
#include "Clases/Gpio/Gpio_Interrupt.h"
//#include "Clases/I2c/i2cdev.h"
//#include "Clases/Pwm/pwm.h"
#include "Clases/Timer/timer.h"
#include "Clases/Uart/uart.h"

#include "Clases/Control/car.h"
#include "Clases/Control/pid.h"
#include "Clases/Sensores/Encoder/encoder.h"

gpio Cam_En(Ddr_Cam, Port_Cam, Pin_Cam);
gpio Wifly_En(Ddr_Wifly, Port_Wifly, Pin_Wifly);
gpio Led(Ddr_Led, Port_Led, Pin_Led);

double alpha;

encoder Enc2(&alpha, Ddr_Enc2, Port_Enc2, In_Enc2, Imsk_Enc2, Pin_Enc2A, Pin_Enc2B);
encoder Enc1(&alpha, Ddr_Enc1, Port_Enc1, In_Enc1, Imsk_Enc1, Pin_Enc1A, Pin_Enc1B);

uint16_t voltaje;

int16_t Right_Speed;
int16_t Left_Speed;

double Ref_Straight;
double Ref_Turn;

double Kd_Straight;
double Kd_Turn;

double Ref_Vel;
double Kp;
double Ki;
double Kd;
pid Right_pid(&Ref_Vel, &Kp, &Kd, &Ki);
pid Left_pid(&Ref_Vel, &Kp, &Kd, &Ki);

double Ref_master;
double Kp_master;
double Ki_master;
double Kd_master;
pid Master_pid(&Ref_master, &Kp_master, &Kd_master, &Ki_master);

bool IsInit;
bool IsMoving;
bool IsTurn;

void wait_wifly()
{	
	_delay_ms(5000);
		
	do
	{
		Uart::Rx = Uart::Receive();	
	} while (Uart::Rx != 'h');
}

void var_init()
{
	voltaje = 0;
	
	Cam_En.output();
	Wifly_En.output();
	Led.output();
	
	//Cam_En.low();
	Cam_En.high();
	Wifly_En.low();		//logica invertida -> high(off) - low(on)
	Led.low();
	
	alpha = 0.7;
	
	Right_Speed = 0;
	Left_Speed = 0;
	
	Ref_Straight = 150; // 300
	Ref_Turn = 50;		// 100
	
	Kd_Straight = 0.001f;
	Kd_Turn = 0.05f;
	
	Ref_Vel = 0; //rpm
	//Kp = 0.1f;
	//Ki = 0.01f;
	//Kd = 0.001f;
	Kp = 0.025f;
	Ki = 0.005f;
	Kd = 0.0005f;
	
	Ref_master = 0;
	Kp_master = 0.1f;
	Ki_master = 0.01f;
	Kd_master = 0.001f;
	
	IsInit = false;
	IsMoving = false;
	IsTurn = false;
}

int main(void)
{
	cli();
	var_init();
	Uart::setup(BAUD_RATE, F_CPU);
	
	wait_wifly();
	
	Enc1.Set_enable(true);
	Enc2.Set_enable(true);
	
	timer1::setup();
	
	adc::setup(true, Pin_Adc);
	
	Uart::Send_string("Loop Begin\n");
	
	sei();
	
	while(1)
	{
		_delay_us(1);
		
#pragma region Reception
		
		if(Uart::IsRx)
		{
			if(IsInit)
			{
				if(Uart::Rx == 'W' || Uart::Rx == 'S')
				{
					timer1::start();
					Ref_Vel = Ref_Straight;
					Kd = Kd_Straight;
					car::direction(Uart::Rx);
					IsMoving = true;
					IsTurn = false;
				}
				
				else if(Uart::Rx == 'A' || Uart::Rx == 'D')
				{
					timer1::start();
					Ref_Vel = Ref_Turn;
					Kd = Kd_Straight;
					car::direction(Uart::Rx);
					IsTurn = true;
					IsMoving = false;
				}
				
				else if(Uart::Rx == 'G')
				{
					timer1::stop();
					Ref_Vel = 0;
					car::direction(Uart::Rx);
					IsMoving = false;
					IsTurn = false;
					
					Enc1.reset();
					Enc2.reset();
					
					Right_pid.restart_errors();
					Right_Speed = 255;
					
					Left_pid.restart_errors();
					Left_Speed = 255;
				}
				
				else if (Uart::Rx == 'm')
				{
					adc::start();
				}
				
				else if (Uart::Rx == 'l')
				{
					Uart::Send_string("Live\n");
				}
				
				else if (Uart::Rx == 'o')
				{
					timer1::stop();
					//Cam_En.low();
					//Wifly_En.high();	//logica invertida -> high(off) - low(on)
					car::off();
					IsInit = false;
					Gpio_Interrupt::Int_PortB_enable(false);				// for Enc2
					Gpio_Interrupt::Int_PortD_enable(false);				// for Enc1
					IsMoving = false;
					IsTurn = false;
					
					Enc1.reset();
					Enc2.reset();
					
					Right_pid.restart_errors();
					Right_Speed = 255;
					
					Left_pid.restart_errors();
					Left_Speed = 255;
					
					Led.low();
				}
				else
				{
					sprintf(Uart::Tx, "The %c char is not in the list\n", Uart::Rx);
					Uart::Send_string(Uart::Tx);
				}
			}
			
			else
			{
				if (Uart::Rx == 's')
				{
					//Cam_En.high();
					IsInit = true;
					//Wifly_En.low();	//logica invertida -> high(off) - low(on)
					car::setup();
					Gpio_Interrupt::Int_PortB_enable(true);				// for Enc2
					Gpio_Interrupt::Int_PortD_enable(true);				// for Enc1
					Led.high();
					IsMoving = false;
				}
				else
				{
					sprintf(Uart::Tx, "The %c char is not in the list\n", Uart::Rx);
					Uart::Send_string(Uart::Tx);
				}
			}
			
			Uart::reset_variable();
		}
		
		#pragma endregion Reception
		
		if(IsInit)
		{		
			if(IsMoving || IsTurn)
			{
				Enc1.update();
				Enc2.update();
				
				Enc1.cal_vel();
				Enc2.cal_vel();

				Master_pid.calc(Enc1.new_vel - Enc2.new_vel, Enc1.diff_time);				
				Right_pid.calc(Enc1.new_vel - Master_pid.E, Enc1.diff_time);
				Left_pid.calc(Enc2.new_vel + Master_pid.E, Enc2.diff_time);
				
				Right_Speed += Right_pid.E;
				if (Right_Speed > 255)		Right_Speed = 255;
				else if (Right_Speed < 0)	Right_Speed = 0;
				
				Left_Speed += Left_pid.E;
				if (Left_Speed > 255)		Left_Speed = 255;
				else if (Left_Speed < 0)	Left_Speed = 0;
				
				car::en_right(uint8_t(Right_Speed & 0xff));
				car::en_left(uint8_t(Left_Speed & 0xff));
				
				//Uart::Send_int(Enc1.diff_time);
				_delay_ms(1);		
			}
		}		
	}
}

ISR(BADISR_vect)
{
	Uart::Send_string("interrupt not handle\n");
}

ISR (ADC_vect)
{
	adc::Interrupt_Handler(&voltaje);
	//Uart::Send_Word(voltaje); // revisar que se envie los datos por wifly (posible falta de "\n")
	Uart::Send_int(voltaje);
}

ISR (USART_RX_vect)
{
	Uart::Rx_Interrupt_Handler();
}

ISR (TIMER1_COMPA_vect)
{
	timer1::CompareMatchA_interrupt_handler();
}

ISR(PCINT0_vect)
{
	Enc2.Interrupt_Count_Handler();
	Gpio_Interrupt::Int_PortB_ClearFlag();
}

ISR(PCINT2_vect)
{
	Enc1.Interrupt_Count_Handler();
	Gpio_Interrupt::Int_PortD_ClearFlag();
}
