/*
 * adc.cpp
 *
 * Created: 09/12/2014 17:08:31
 *  Author: Reta
 */ 

// in auto-trigger mode, the adc use 38.5 cycles (308 uS) for the first conversion and 15.5 cycles (124 uS) of the adc clock for the conversion
// to this time we have to add the time for do the Interrupt_Handler method

#include "adc.h"

adc::adc()
{
	
}

void adc::setup(bool IsAVcc_Ref, uint8_t pin)
{
	ADMUX  = pin;																	// use ADC0
	
	if(IsAVcc_Ref)
		ADMUX  |=  (1 << REFS0);												// use AVcc as the reference
	else
		ADMUX  |=  (1 << REFS1) | (1 << REFS0);									// Internal 1.1V
	
	ADMUX  &= ~(1 << ADLAR);													// Right adjust for 12 bit resolution
	//ADMUX  |= (1 << ADLAR);													// Left adjust for 10 bit resolution
	
	ADCSRA |=  (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);						// 128 pre-scale for 16Mhz = 125Khz -> 8 uS every cycles (adc clock must be between 50k - 200k)
	
	ADCSRA &= ~(1 << ADATE);													// Single Conversion
	//ADCSRA |= (1 << ADATE);														// Conversion Triggered by setup ADTS2:0
	
	//ADCSRB |= (1 << ADTS2) | (1 << ADTS0);										// Conversion triggered by Timer 1 B Compare
	//ADCSRB |= (1 << ADTS2) | (1 << ADTS1);										// Conversion triggered by Timer 1 Overflow
	
	ADCSRA |=  (1 << ADIE);														// Enable Interrupts	
	
	DIDR0  |=  (1<<ADC1D) | (1<<ADC2D) | (1<<ADC3D) | (1<<ADC4D) | (1<<ADC5D);				// Disable adc input pin from 1 to 5;
}

void adc::off()
{
	ADCSRA = 0;
	ADCSRB = 0;
}

void adc::start()
{
	ADCSRA |= (1 << ADSC)|(1 << ADEN);					  // Start and Enable the ADC conversion
	//ADCSRA |= (1 << ADEN);									  // Enable the ADC conversion
}

void adc::stop()
{
	ADCSRA &= ~((1 << ADSC)|(1 << ADEN));				  // Stop the ADC conversion
	//ADCSRA &= ~(1 << ADEN);								  // Stop the ADC conversion
}

void adc::Interrupt_Handler(uint16_t * sen)
{
	uint16_t ADCval;
	
	//we have to read ADCL first, because after read ADCH causes to ADC update.	
	ADCval = ADCL;
	ADCval = (ADCH << 8) + ADCval;    // ADCH is read so ADC can be updated again

	*sen = ADCval;			// put ADCvalue in the register you use for ADC0 sensor
}

void adc::Interrupt_Handler2(uint16_t * sen1, uint16_t * sen2, volatile bool * IsSenReady)
{
	//send to byte of the conversion, the high value first
	
	uint16_t ADCval;
	uint8_t tmp;            // temp register for storage of misc data

	tmp = ADMUX;            // read the value of ADMUX register
	tmp &= 0x03;            // AND the first 2 bits (value of ADC pin being used)
	
	//we have to read ADCL first, because after read ADCH causes to ADC update.
	ADCval = ADCL;
	ADCval = (ADCH << 8) + ADCval;    // ADCH is read so ADC can be updated again

	if (tmp == 0)
	{
		*sen1 = ADCval;			// put ADCvalue in the register you use for ADC0 sensor
		ADMUX++;				// add 1 to ADMUX to go to the next sensor
	}
	
	else if (tmp == 1)
	{
		*sen2 = ADCval;			// put ADCvalue in the register you use for ADC1 sensor
		*IsSenReady = true;
		ADMUX &= 0xFC;			//  clear the last 2 bits to reset the mux to ADC0
	}
}


	
