﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;
using System.Diagnostics;

using System.IO;
using Foscam_Mpeg;
using System.Threading;
using Interfaz_v1.Class.Events;


namespace Interfaz_v1.Class.Camera
{   
    //clase para ocupar una camara ip
    public class Camera
    {   
        // Variables de estado
        public bool IsReady;

        // Camera ip login info
        private string admin_user;
        private string oper_user;
        private string pass_user;

        private MyTcpClient Camara_Ip = null;
        private Sound Audio = null;

        public event NewVideoFrameEventHandler VideoFrame;
        public event BoolEventHandler ConnectEvent;
        public event BoolEventHandler FoundEvent;

        // Constructor para inicializar parametros
        public Camera()
        {
            IsReady = false;

            extract_file_info();

            Camara_Ip = new MyTcpClient(admin_user, pass_user); 
            //Audio = new Sound();

            Camara_Ip.FoundEvent += Camara_Ip_FoundEvent;
            Camara_Ip.ConnectEvent += Camara_Ip_ConnectEvent;
        }



        // Metodo para buscar la camara en la red local
        public void Search()
        {            
            Camara_Ip.Search();                
        }

        // Metodo para setear todo lo necasario para mostrar una camara ip o web
        public void Connect()
        {
            if (Camara_Ip.IsFounded)
            {
                Camara_Ip.Oprt_Init();                
            }
        }

        // Metodo para iniciar la adquisicion de la camara
        // Falta verificar coneccion
        public void Setup()
        {
            if (Camara_Ip.IsOperating)
            {
                Camara_Ip.NewVideoFrame += Update_Image;
                Camara_Ip.Video_Init();

                Camara_Ip.Str_Init(Camara_Ip.Id);

                //Audio.Init();

                //Audio.Play_Start(Camara_Ip);
                //Camara_Ip.AudioIn_Init();

                //Camara_Ip.AudioOut_Init();
                //Audio.Talk_Start();

                // Move to the center
                Camara_Ip.Move_Center();                

                // Esperar que se mueva al centro
                Thread.Sleep(10000);

                Camara_Ip.Move_Down();
                Thread.Sleep(3000);
                Camara_Ip.Move_Stop();

                // turn on leds infrarrojos
                Camara_Ip.IR_On();

                // set presset 1
                Camara_Ip.Presset_Set_1();

                IsReady = true;
            }
        }

        // Event handler para cada new frame
        private void Update_Image(Foscam_Mpeg.Events.NewVideoFrameEventArgs e)
        {
            if (VideoFrame != null) VideoFrame(new NewVideoFrameEventArgs(e.frame));
        }

        private void Camara_Ip_FoundEvent(Foscam_Mpeg.Events.BoolEventArgs e)
        {
            if (FoundEvent != null) FoundEvent(new BoolEventArgs(e.Bool));
        }

        private void Camara_Ip_ConnectEvent(Foscam_Mpeg.Events.BoolEventArgs e)
        {
            if (ConnectEvent != null) ConnectEvent(new BoolEventArgs(e.Bool));
        }

        // Metodo para cerrar la camara e hilos
        public void Closing()
        {
            if (Camara_Ip.IsFounded)
            {
                if (Camara_Ip.IsVideoIn)
                {
                    Camara_Ip.NewVideoFrame -= Update_Image;
                    Camara_Ip.Video_Stop();
                }

                if (Camara_Ip.IsAudioIn)
                {
                    Camara_Ip.AudioIn_Stop();
                    Audio.Play_Stop(Camara_Ip);
                }

                if (Camara_Ip.IsAudioOut)
                {
                    Audio.Talk_Stop();
                    Camara_Ip.AudioOut_Stop(Audio);
                }

                if (Audio != null)
                {
                    Audio.Close();
                    Audio = null;
                }

                if (Camara_Ip.IsStreaming)
                {
                    Camara_Ip.Str_Close();
                }

                if (Camara_Ip.IsMove)
                {
                    Camara_Ip.Move_Stop();
                }

                if (Camara_Ip.IsOperating)
                {
                    Camara_Ip.Oprt_Close();
                }

                if (Camara_Ip != null)
                {
                    Camara_Ip = null;
                }
            }    
  
        }

        public void Move(string msj)
        {
            if (IsReady && Camara_Ip.IsOperating)
            {
                switch (msj)
                {
                    case "Up":
                        Camara_Ip.Move_Up();
                        break;
                    case "Down":
                        Camara_Ip.Move_Down();
                        break;
                    case "Right":
                        Camara_Ip.Move_Right();
                        break;
                    case "Left":
                        Camara_Ip.Move_Left();
                        break;
                    case "Stop":
                        Camara_Ip.Move_Stop();
                        break;
                    case "Preset_1":
                        Camara_Ip.Presset_Go_1();
                        break;
                    default:
                        Debug.WriteLine("Cannot Move in that way");
                        break;
                }
            }
        }

        private void extract_file_info()
        {
            try
            {
                string info_setup = File.ReadAllText(@"Setup\Setup_information.txt", Encoding.ASCII);

                int a = info_setup.IndexOf("Admin_User: ");
                int b = info_setup.IndexOf(" ", a);
                int c = info_setup.IndexOf("\n", b);

                admin_user = info_setup.Substring(b + 1, c - 1 - b);

                a = info_setup.IndexOf("Operator_User: ");
                b = info_setup.IndexOf(" ", a);
                c = info_setup.IndexOf("\n", b);

                oper_user = info_setup.Substring(b + 1, c - 1 - b);

                a = info_setup.IndexOf("Password: ");
                b = info_setup.IndexOf(" ", a);
                c = info_setup.IndexOf("\n", b);

                pass_user = info_setup.Substring(b + 1, c - 1 - b);
            }

            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

    }
}
