﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.IO.Ports;
using System.Diagnostics;
using Interfaz_v1.Class.Events;

// Revisar Live function (isLive parameter)
// Revisar battery Request function (listo)

namespace Interfaz_v1.Class.Comunication
{
    // Clase para enviar y recibir información a través del protocolo UDP
    // especialmente construido para usarlo con el módulo WiFly
    public class Wifly
    {
        // variables para guardar la ip dinamica del modulo Wifly
        private IPAddress Wifly_ip;
         
        private int Wifly_port;         // puerto con el cual wiFly recive la info
        private int Host_port;          // puerto con el cual el wifly envia la info
        private int Broadcast_port;     // puerto con el cual el wifly envia el brodcast

        private UdpClient Receive_client;
        private UdpClient Send_client;

        public bool IsFounded;
        public bool IsConnected;
        private bool IsLive;

        private bool SendBatteryRequest;
        private bool SendLiveRequest;

        public event BoolEventHandler FoundEvent;
        public event BoolEventHandler ConnectionEvent;
        public event BatteryEventHandler BatteryResponseEvent;

        // Constructor de la Clase Wifly
        // setea los parametros necesarios para efectuar la comunciacion
        public Wifly()
        {
            Host_port = 4000; // elegido al azar
            Broadcast_port = 10000;

            IsFounded = false;
            IsConnected = false;

            IsLive = false;
            SendBatteryRequest = false;
            SendLiveRequest = false;
        }

        // Metodo para poder setear los parametros necesarios para la comunicación con el Wifly
        // Con este método es posible la comunicación sin saber la ip ni el puerto con que envia el modulo Wifly
        public void Search()
        {
            try
            {   // Se construye el objeto udpClient con el puerto por el cual se está enviando el broadcast del módulo Wifly
                UdpClient udpClient = new UdpClient(Broadcast_port);
                udpClient.Client.ReceiveTimeout = 5000;

                // El objeto IpEndPoint seteado de esta forma nos permitira leer datos enviados por cualquier fuente
                IPEndPoint RemoteIpEndPoint = new IPEndPoint(IPAddress.Any, Broadcast_port);

                // Se bloquea hasta que se reciva el mensaje de lareferencia RemoteIpEndPoint
                byte[] receiveBytes = udpClient.Receive(ref RemoteIpEndPoint);

                // Mensaje en la consola para mostrar la ip y puerto del modulo WiFly
                //My_Form.SetText_enviado("Wifly: This message was sent from " +
                //                            RemoteIpEndPoint.Address.ToString() +
                //                            " on their port number " +
                //                            RemoteIpEndPoint.Port.ToString() + "\n");

                // Se cierra el objeto UdpClient
                udpClient.Close();

                // Se guarda el valor de la ip y del puerto ocupado por el módulo Wifly
                Wifly_ip = RemoteIpEndPoint.Address;
                Wifly_port = RemoteIpEndPoint.Port;

                IsFounded = true;
            }

            // necesario para coger las excepciones sin que se caiga el programa
            catch (Exception ex)
            {
                //My_Form.SetText_enviado(ex.ToString() + "\n");
                Debug.WriteLine(ex.ToString());
            }

            finally
            {
                OnFoundEvent(new BoolEventArgs(IsFounded));
            }
        }

        public void Connect()
        {
            try
            {
                // Se construye el objeto udpClient con el puerto por el cual el módulo Wifly esta enviando
                Send_client = new UdpClient();
                Send_client.Client.BeginConnect(Wifly_ip.ToString(), Wifly_port, new AsyncCallback(ConnectCallback), Send_client);
            }

            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
                //My_Form.SetText_enviado(ex.ToString() + "\n");
            }
        }

        public void ConnectCallback(IAsyncResult ar)
        {
            try
            {
                UdpClient Client = (UdpClient)ar.AsyncState;
                Client.Client.EndConnect(ar);

                if (Client.Client.Connected) IsConnected = true;
            }

            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
                //My_Form.SetText_enviado(ex.ToString() + "\n");
            }
            finally
            {
                OnConnectionEvent(new BoolEventArgs(IsConnected));
            }
        }

        public void Setup()
        {
            //IsLive = true;
            Begin_Receive();
            Thread.Sleep(200);
            Send("h");
            Thread.Sleep(200);
            Send("s");
        }

        public void Send(string msj)
        {
            if (IsConnected)
            {
                try
                {
                    // se envia el mensaje por la conección anteriormente abierta
                    Byte[] sendBytes = Encoding.ASCII.GetBytes(msj);

                    // send the message, the destination is defined by the call to .Connect()
                    Send_client.BeginSend(sendBytes, sendBytes.Length, new AsyncCallback(SendCallback), Send_client);

                    // Se imprime en la consola lo enviado
                    Debug.WriteLine("Sended: " + msj);
                    //My_Form.SetText_enviado(msj + "\n");
                }

                catch (Exception ex)
                {
                    Debug.WriteLine(ex.ToString());
                    //My_Form.SetText_enviado(ex.ToString() + "\n");
                }
            }
        }

        public void SendCallback(IAsyncResult ar)
        {
            try
            {
                UdpClient Client = (UdpClient)ar.AsyncState;
                Client.EndSend(ar);
            }

            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
                //My_Form.SetText_enviado(ex.ToString() + "\n");
            }
        }

        private struct udpstate
        {
            public readonly IPEndPoint ip;
            public readonly UdpClient udp;

            public udpstate(IPEndPoint i, UdpClient u)
            {
                ip = i;
                udp = u;
            }
        }

        public void Begin_Receive()
        {
            try
            {
                // Se construye el objeto udpClient con el puerto por el cual el módulo envía el mensaje
                Receive_client = new UdpClient(Host_port);

                /// El objeto IpEndPoint seteado de esta forma nos permitira leer datos enviados desde la siguiente ip y puerto
                IPEndPoint RemoteIpEndPoint = new IPEndPoint(Wifly_ip, Host_port); // host_port

                udpstate state = new udpstate(RemoteIpEndPoint, Receive_client);

                Receive_client.BeginReceive(new AsyncCallback(ReceiveCallback), state);
            }

            // necesario para coger las excepciones sin que se caiga el programa
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
                //My_Form.SetText_enviado(ex.ToString() + "\n");
            }
        }
        
        private void ReceiveCallback(IAsyncResult ar)
        {
            try
            {
                udpstate state = (udpstate)ar.AsyncState;
                UdpClient udp = (UdpClient)state.udp;
                IPEndPoint ip = (IPEndPoint)state.ip;

                Byte[] receiveBytes = udp.EndReceive(ar, ref ip);
                string receiveString = Encoding.ASCII.GetString(receiveBytes);

                Debug.Write("recivi:" + receiveString);

                if(SendBatteryRequest)
                {
                    OnBatteryResponseEvent(new BatteryEventArgs(receiveString));
                    SendBatteryRequest = false;
                }

                else if (SendLiveRequest)
                {
                    if (receiveString.Equals("Live")) IsLive = true;
                    else IsLive = false;

                }

                //My_Form.SetText_recivido(receiveString);

                udp.BeginReceive(new AsyncCallback(ReceiveCallback), state);                
            }

            // necesario para coger las excepciones sin que se caiga el programa
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
                //My_Form.SetText_enviado(ex.ToString() + "\n");
            }
        }
        
        public void closing()
        {
            if(IsConnected)
            {
                Move("Stop");
                Thread.Sleep(200);
                Send("o");
                Thread.Sleep(200);
            }

            if(Send_client != null)
            {
                Send_client.Close();
            }

            if (Receive_client != null) // Receive_Thread.IsAlive
            {
                Receive_client.Close();
            }
        }

        public void Move(string msj)
        {
            switch (msj)
            {
                case "Up":
                    Send("W");
                    break;
                case "Down":
                    Send("S");
                    break;
                case "Right":
                    Send("D");
                    break;
                case "Left":
                    Send("A");
                    break;
                case "Stop":
                    Send("G");
                    break;
                default:
                    Debug.WriteLine("Cannot Move in that way");
                    break;
            }
        }

        public void BatteryRequest()
        {            
            Send("m");
            SendBatteryRequest = true;
        }
        
        private void OnConnectionEvent(BoolEventArgs e)
        {
            if (ConnectionEvent != null) ConnectionEvent(e);
        }

        private void OnFoundEvent(BoolEventArgs e)
        {
            if (FoundEvent != null) FoundEvent(e);
        }     
                
        protected virtual void OnBatteryResponseEvent(BatteryEventArgs e)
        {
            if (BatteryResponseEvent != null) BatteryResponseEvent(e);
        }

    }

}

